# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::Advisor::Mgmt::V2016_07_12_preview
  #
  # REST APIs for Azure Advisor
  #
  class Suppressions
    include MsRestAzure

    #
    # Creates and initializes a new instance of the Suppressions class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [AdvisorManagementClient] reference to the AdvisorManagementClient
    attr_reader :client

    #
    # Obtains the details of a suppression.
    #
    # @param resource_uri [String] The fully qualified Azure Resource Manager
    # identifier of the resource to which the recommendation applies.
    # @param recommendation_id [String] The recommendation ID.
    # @param name [String] The name of the suppression.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [SuppressionContract] operation results.
    #
    def get(resource_uri, recommendation_id, name, custom_headers:nil)
      response = get_async(resource_uri, recommendation_id, name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Obtains the details of a suppression.
    #
    # @param resource_uri [String] The fully qualified Azure Resource Manager
    # identifier of the resource to which the recommendation applies.
    # @param recommendation_id [String] The recommendation ID.
    # @param name [String] The name of the suppression.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_uri, recommendation_id, name, custom_headers:nil)
      get_async(resource_uri, recommendation_id, name, custom_headers:custom_headers).value!
    end

    #
    # Obtains the details of a suppression.
    #
    # @param resource_uri [String] The fully qualified Azure Resource Manager
    # identifier of the resource to which the recommendation applies.
    # @param recommendation_id [String] The recommendation ID.
    # @param name [String] The name of the suppression.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_uri, recommendation_id, name, custom_headers:nil)
      fail ArgumentError, 'resource_uri is nil' if resource_uri.nil?
      fail ArgumentError, 'recommendation_id is nil' if recommendation_id.nil?
      fail ArgumentError, 'name is nil' if name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{resourceUri}/providers/Microsoft.Advisor/recommendations/{recommendationId}/suppressions/{name}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceUri' => resource_uri,'recommendationId' => recommendation_id,'name' => name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Advisor::Mgmt::V2016_07_12_preview::Models::SuppressionContract.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Enables the snoozed or dismissed attribute of a recommendation. The snoozed
    # or dismissed attribute is referred to as a suppression. Use this API to
    # create or update the snoozed or dismissed status of a recommendation.
    #
    # @param resource_uri [String] The fully qualified Azure Resource Manager
    # identifier of the resource to which the recommendation applies.
    # @param recommendation_id [String] The recommendation ID.
    # @param name [String] The name of the suppression.
    # @param suppression_contract [SuppressionContract] The snoozed or dismissed
    # attribute; for example, the snooze duration.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [SuppressionContract] operation results.
    #
    def create(resource_uri, recommendation_id, name, suppression_contract, custom_headers:nil)
      response = create_async(resource_uri, recommendation_id, name, suppression_contract, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Enables the snoozed or dismissed attribute of a recommendation. The snoozed
    # or dismissed attribute is referred to as a suppression. Use this API to
    # create or update the snoozed or dismissed status of a recommendation.
    #
    # @param resource_uri [String] The fully qualified Azure Resource Manager
    # identifier of the resource to which the recommendation applies.
    # @param recommendation_id [String] The recommendation ID.
    # @param name [String] The name of the suppression.
    # @param suppression_contract [SuppressionContract] The snoozed or dismissed
    # attribute; for example, the snooze duration.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_with_http_info(resource_uri, recommendation_id, name, suppression_contract, custom_headers:nil)
      create_async(resource_uri, recommendation_id, name, suppression_contract, custom_headers:custom_headers).value!
    end

    #
    # Enables the snoozed or dismissed attribute of a recommendation. The snoozed
    # or dismissed attribute is referred to as a suppression. Use this API to
    # create or update the snoozed or dismissed status of a recommendation.
    #
    # @param resource_uri [String] The fully qualified Azure Resource Manager
    # identifier of the resource to which the recommendation applies.
    # @param recommendation_id [String] The recommendation ID.
    # @param name [String] The name of the suppression.
    # @param suppression_contract [SuppressionContract] The snoozed or dismissed
    # attribute; for example, the snooze duration.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_async(resource_uri, recommendation_id, name, suppression_contract, custom_headers:nil)
      fail ArgumentError, 'resource_uri is nil' if resource_uri.nil?
      fail ArgumentError, 'recommendation_id is nil' if recommendation_id.nil?
      fail ArgumentError, 'name is nil' if name.nil?
      fail ArgumentError, 'suppression_contract is nil' if suppression_contract.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::Advisor::Mgmt::V2016_07_12_preview::Models::SuppressionContract.mapper()
      request_content = @client.serialize(request_mapper,  suppression_contract)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = '{resourceUri}/providers/Microsoft.Advisor/recommendations/{recommendationId}/suppressions/{name}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceUri' => resource_uri,'recommendationId' => recommendation_id,'name' => name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Advisor::Mgmt::V2016_07_12_preview::Models::SuppressionContract.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Enables the activation of a snoozed or dismissed recommendation. The snoozed
    # or dismissed attribute of a recommendation is referred to as a suppression.
    #
    # @param resource_uri [String] The fully qualified Azure Resource Manager
    # identifier of the resource to which the recommendation applies.
    # @param recommendation_id [String] The recommendation ID.
    # @param name [String] The name of the suppression.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def delete(resource_uri, recommendation_id, name, custom_headers:nil)
      response = delete_async(resource_uri, recommendation_id, name, custom_headers:custom_headers).value!
      nil
    end

    #
    # Enables the activation of a snoozed or dismissed recommendation. The snoozed
    # or dismissed attribute of a recommendation is referred to as a suppression.
    #
    # @param resource_uri [String] The fully qualified Azure Resource Manager
    # identifier of the resource to which the recommendation applies.
    # @param recommendation_id [String] The recommendation ID.
    # @param name [String] The name of the suppression.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def delete_with_http_info(resource_uri, recommendation_id, name, custom_headers:nil)
      delete_async(resource_uri, recommendation_id, name, custom_headers:custom_headers).value!
    end

    #
    # Enables the activation of a snoozed or dismissed recommendation. The snoozed
    # or dismissed attribute of a recommendation is referred to as a suppression.
    #
    # @param resource_uri [String] The fully qualified Azure Resource Manager
    # identifier of the resource to which the recommendation applies.
    # @param recommendation_id [String] The recommendation ID.
    # @param name [String] The name of the suppression.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def delete_async(resource_uri, recommendation_id, name, custom_headers:nil)
      fail ArgumentError, 'resource_uri is nil' if resource_uri.nil?
      fail ArgumentError, 'recommendation_id is nil' if recommendation_id.nil?
      fail ArgumentError, 'name is nil' if name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{resourceUri}/providers/Microsoft.Advisor/recommendations/{recommendationId}/suppressions/{name}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceUri' => resource_uri,'recommendationId' => recommendation_id,'name' => name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 204
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Retrieves the list of snoozed or dismissed suppressions for a subscription.
    # The snoozed or dismissed attribute of a recommendation is referred to as a
    # suppression.
    #
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array] operation results.
    #
    def list(custom_headers:nil)
      response = list_async(custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Retrieves the list of snoozed or dismissed suppressions for a subscription.
    # The snoozed or dismissed attribute of a recommendation is referred to as a
    # suppression.
    #
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(custom_headers:nil)
      list_async(custom_headers:custom_headers).value!
    end

    #
    # Retrieves the list of snoozed or dismissed suppressions for a subscription.
    # The snoozed or dismissed attribute of a recommendation is referred to as a
    # suppression.
    #
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Advisor/suppressions'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = {
              client_side_validation: true,
              required: false,
              serialized_name: 'parsed_response',
              type: {
                name: 'Sequence',
                element: {
                    client_side_validation: true,
                    required: false,
                    serialized_name: 'SuppressionContractElementType',
                    type: {
                      name: 'Composite',
                      class_name: 'SuppressionContract'
                    }
                }
              }
            }
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

  end
end
